require_relative 'game_turn'
module StudioGame
    class Game

        def initialize (title)
            @title = title
            @players = []
        end
        def high_score_entry(player)
            "\n#{player.name.ljust(20,".")} #{player.score}"
        end
        def load_players(from_file)
            File.readlines(from_file).each do |line|
                add_player(Player.from_csv(line))
            end
        end
        def save_high_scores(to_file="high_scores.txt")
            File.open(to_file, "w") do |file|
                file.puts "#{@title} High scores :"
                @players.sort.each do |player|
                    file.puts high_score_entry(player)
                end
            end
        end
        def add_player(player)
            @players << player
        end
        def print_name_and_health(player)
            puts "#{player.name} (#{player.health})"
          end
    
          def total_points
            @players.reduce(0) {|sum, player| sum + player.points}
        end 
    
        def print_stats
            strong_players, wimpy_players =  @players.partition { |player| player.strong? }
            puts "\n#{@title} Statistics"
            puts "\n#{strong_players.count} strong players:"
            strong_players.each do |player|
                print_name_and_health(player)
            end
            puts "\n#{wimpy_players.count} wimpy players:"
            wimpy_players.each do |player|
                print_name_and_health(player)
            end
            @players.each do |player|
                puts "\n#{player.name}'s point totals:"
                player.each_found_treasure do |treasure|
                    puts "#{treasure.points} total #{treasure.name} points"
                end
                puts "#{player.points} grand total points"
            end
            @players.sort.each do |player|
                puts high_score_entry(player)
            end
            puts "#{total_points} points from treasures found"
        end
    
        def play(rounds)
            1.upto(rounds) do |round|
                if block_given?
                    break if yield
                else
                    puts "\nRound #{round}"
                    puts "There are #{@players.size} players in the game :"
                    @players.each do|player|
                        puts player
                    end
                    treasures = TreasureTrove::TREASURES
                    puts "\nThere are #{treasures.size} treasures to be found :"
                    treasures.each do |treasure| 
                        puts "A #{treasure.name} is worth #{treasure.points} points"
                    end
                    @players.each do |player|  
                        GameTurn.take_turn(player)
                        puts player
                    end
                end
            end
        end
        
    end
    
    
end

